package com.perforce.p4asset;

import java.util.Collections;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.eclipse.jetty.security.ConstraintMapping;
import org.eclipse.jetty.security.ConstraintSecurityHandler;
import org.eclipse.jetty.security.HashLoginService;
import org.eclipse.jetty.security.LoginService;
import org.eclipse.jetty.security.authentication.BasicAuthenticator;
import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.server.ServerConnector;
import org.eclipse.jetty.servlet.ServletContextHandler;
import org.eclipse.jetty.servlet.ServletHolder;
import org.eclipse.jetty.util.security.Constraint;
import org.glassfish.jersey.servlet.ServletContainer;

public class HttpServlet {

	private static final Logger logger = LogManager.getLogger(HttpServlet.class
			.getName());

	private Server server;

	public HttpServlet() {

		PerforceConnectionConfig p4config = new PerforceConnectionConfig(
				"localhost:1666");

		logger.info("Starting HTTP server ...");
		server = new Server();

		// setup connector
		ServerConnector connector = new ServerConnector(server);
		connector.setPort(8080);
		server.addConnector(connector);

		// setup BASIC auth
		LoginService loginService = new PerforceLoginService(p4config);
		LoginService loginService2 = new HashLoginService("MyRealm",
				"src/test/resources/realm.properties");
		server.addBean(loginService);

		ConstraintSecurityHandler security = new ConstraintSecurityHandler();
		server.setHandler(security);

		Constraint constraint = new Constraint();
		constraint.setName(Constraint.__BASIC_AUTH);
		constraint.setAuthenticate(true);
		constraint.setRoles(new String[] { "user", "admin" });

		ConstraintMapping mapping = new ConstraintMapping();
		mapping.setPathSpec("/*");
		mapping.setConstraint(constraint);

		security.setConstraintMappings(Collections.singletonList(mapping));
		security.setAuthenticator(new BasicAuthenticator());
		security.setLoginService(loginService);

		// set handler with security wrapping
		ServletContextHandler context = new ServletContextHandler(
				ServletContextHandler.SESSIONS);
		context.setContextPath("/");
		security.setHandler(context);

		// add servlet to context
		ServletHolder jerseyServlet = new ServletHolder("api",
				ServletContainer.class);
		jerseyServlet.setInitParameter(
				"jersey.config.server.provider.packages",
				"com.perforce.p4asset.controllers");
		context.addServlet(jerseyServlet, "/*");

		try {
			server.start();
			server.dump(System.err);
			server.join();
		} catch (Exception e) {
			logger.error(e);
		}
	}
}
