package com.perforce.p4asset;

import java.util.Properties;

import com.perforce.p4java.PropertyDefs;
import com.perforce.p4java.impl.mapbased.rpc.RpcPropertyDefs;
import com.perforce.p4java.server.IOptionsServer;
import com.perforce.p4java.server.ServerFactory;

public class PerforceConnectionConfig {

	private final String p4port;
	private final boolean ssl;
	private final String uri;

	public PerforceConnectionConfig(String address) {
		ssl = address.startsWith("ssl");
		if (ssl) {
			p4port = address.substring(address.indexOf(":"));
			uri = "p4javassl://" + p4port;
		} else {
			p4port = address;
			uri = "p4java://" + p4port;
		}
	}

	public String getP4port() {
		return p4port;
	}

	public boolean isSsl() {
		return ssl;
	}

	public String getUri() {
		return uri;
	}

	public IOptionsServer getConnection() throws Exception {
		Properties props = System.getProperties();

		// Identify ourselves in server log files.
		Identifier id = new Identifier();
		props.put(PropertyDefs.PROG_NAME_KEY, id.getProduct());
		props.put(PropertyDefs.PROG_VERSION_KEY, id.getVersion());

		// Allow p4 admin commands.
		props.put(RpcPropertyDefs.RPC_RELAX_CMD_NAME_CHECKS_NICK, "true");

		// disable timeout for slow servers / large db lock times
		props.put(RpcPropertyDefs.RPC_SOCKET_SO_TIMEOUT_NICK, "0");

		// Get a server connection
		IOptionsServer iserver = ServerFactory.getOptionsServer(uri, props);
		return iserver;
	}

}
