package com.perforce.p4asset;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.eclipse.jetty.security.DefaultIdentityService;
import org.eclipse.jetty.security.IdentityService;
import org.eclipse.jetty.security.LoginService;
import org.eclipse.jetty.server.UserIdentity;
import org.eclipse.jetty.util.component.AbstractLifeCycle;

import com.perforce.p4java.exception.AccessException;
import com.perforce.p4java.exception.ConfigException;
import com.perforce.p4java.exception.ConnectionException;
import com.perforce.p4java.exception.RequestException;
import com.perforce.p4java.server.IOptionsServer;

public class PerforceLoginService extends AbstractLifeCycle implements
		LoginService {

	private static final Logger logger = LogManager
			.getLogger(PerforceLoginService.class.getName());

	protected IdentityService identityService = new DefaultIdentityService();

	private PerforceConnectionConfig p4config;
	private IOptionsServer iserver;

	PerforceLoginService(PerforceConnectionConfig p4config) {
		this.p4config = p4config;
	}

	@Override
	public String getName() {
		return null;
	}

	@Override
	public UserIdentity login(String username, Object credentials) {
		try {
			iserver = p4config.getConnection();

			// Add trust for SSL connections, if it is not already there
			if (p4config.isSsl()) {
				String serverTrust = iserver.getTrust();
				// TODO just adding trust with no check
				iserver.addTrust(serverTrust);
			}

			// Connect to the server
			iserver.connect();

			// CHARSET is not defined (only for client access)
			if (iserver.getServerInfo().isUnicodeEnabled()) {
				iserver.setCharsetName("utf8");
			}

			// Login Perforce user
			String status = iserver.getLoginStatus();
			if (!status.contains("not necessary")) {
				String password = (String) credentials;
				iserver.login(password);
			}
			return new PerforceUserIdentity(username, (String) credentials);

		} catch (Exception e) {
			logger.error(e);
			return null;
		}
	}

	@Override
	public boolean validate(UserIdentity user) {
		if (user instanceof PerforceUserIdentity) {
			PerforceUserIdentity p4uid = (PerforceUserIdentity) user;
			String p4user = p4uid.getUsername();
			String p4ticket = p4uid.getCredentials();
			if (p4user != null) {
				return p4ticket.equals(p4user);
			}
		}
		return false;
	}

	@Override
	public IdentityService getIdentityService() {
		return this.identityService;
	}

	@Override
	public void setIdentityService(IdentityService identityService) {
		if (isRunning())
			throw new IllegalStateException("Running");
		this.identityService = identityService;
	}

	@Override
	public void logout(UserIdentity user) {
		try {
			iserver.logout();
		} catch (ConnectionException e) {
			logger.error(e);
		} catch (RequestException e) {
			logger.error(e);
		} catch (AccessException e) {
			logger.error(e);
		} catch (ConfigException e) {
			logger.error(e);
		}
	}

}
